@@ -19,9 +19,9 @@
 import collections
 import os
 import re
-import subprocess
 import tempfile
 from gbp.errors import GbpError
+from gbp.git.repository import GitRepository
 
 VALID_DEP3_ENDS = re.compile(r'(?:---|\*\*\*|Index:)[ \t][^ \t]|^diff -|^---')
 
@@ -68,17 +68,38 @@ class Patch(object):
         using I{git mailinfo}
         """
         self.info = {}
+        self.long_desc = ''
+
         body = tempfile.NamedTemporaryFile(prefix='gbp_')
-        pipe = subprocess.Popen("git mailinfo -k '%s' /dev/null 2>/dev/null < '%s'" %
-                                (body.name, self.path),
-                                shell=True,
-                                stdout=subprocess.PIPE).stdout
-        for line in pipe:
-            line = line.decode()
+
+        # No patch yet, file name information only
+        if not os.path.exists(self.path):
+            return
+        # The patch description might contain UTF-8 while the actual patch is ascii.
+        # To unconfuse git-mailinfo stop at the patch separator
+        toparse = []
+        for line in open(self.path, 'rb'):
+            if line == b'---\n':
+                break
+            toparse.append(line)
+
+        out, err, ret = GitRepository.git_inout(command='mailinfo',
+                                                args=['-k', body.name, '/dev/null'],
+                                                input=b''.join(toparse),
+                                                extra_env=None,
+                                                cwd=None,
+                                                capture_stderr=True)
+        if ret != 0:
+            raise GbpError("Failed to read patch header of '%s': %s" %
+                           (self.path, err))
+
+        # Header
+        for line in out.decode().split('\n'):
             if ':' in line:
                 rfc_header, value = line.split(" ", 1)
                 header = rfc_header[:-1].lower()
                 self.info[header] = value.strip()
+        # Body
         try:
             self.long_desc = "".join([l.decode("utf-8", "backslashreplace") for l in body])
         except (IOError, UnicodeDecodeError) as msg:
@@ -169,8 +190,7 @@ class Patch(object):
 class Dep3Patch(Patch):
     def _read_info(self):
         super(Dep3Patch, self)._read_info()
-        if not self.info:
-            self._check_dep3()
+        self._check_dep3()
 
     def _dep3_get_value(self, lines):
         value = []
@@ -191,6 +211,8 @@ class Dep3Patch(Patch):
         """
 
         def add_author(lines):
+            if 'email' in self.info and 'author' in self.info:
+                return 0
             value = self._dep3_get_value(lines).strip()
             m = re.match('(.*)<([^<>]+)>', value)
             if m:
@@ -200,6 +222,8 @@ class Dep3Patch(Patch):
             return 1
 
         def add_subject(lines, long_desc, changes):
+            if 'subject' in self.info:
+                return long_desc, 0
             value = self._dep3_get_value(lines).lstrip()
             if '\n' in value:
                 value, description = value.split('\n', 1)
@@ -208,7 +232,14 @@ class Dep3Patch(Patch):
             self.info['subject'] = value
             return long_desc, changes + 1
 
+        def add_date(lines):
+            if 'date' in self.info:
+                return 0
+            self.info['date'] = self._dep3_get_value(lines).strip()
+            return 1
+
         changes = 0
+        pseudo_headers = ''
         long_desc = self._dep3_get_value(headers.get('long_desc', list()))
 
         for k, v in headers.items():
@@ -216,13 +247,17 @@ class Dep3Patch(Patch):
                 changes += add_author(v)
             elif k in ('subject', 'description'):
                 long_desc, changes = add_subject(v, long_desc, changes)
+            elif k in ['date']:
+                changes += add_date(v)
             elif k == 'long_desc':
                 pass
             else:
-                long_desc += ''.join(v)
+                pseudo_headers += ''.join(v)
                 changes += 1
         if changes:
-            self.long_desc = long_desc + self.long_desc
+            self.long_desc = (pseudo_headers +
+                              ('\n' if pseudo_headers else '') +
+                              long_desc + self.long_desc)
 
     def _check_dep3(self):
         """
@@ -258,8 +293,8 @@ class PatchSeries(list):
     """
     A series of L{Patch}es as read from a quilt series file).
     """
-    comment_re = re.compile('\s+#.*$')
-    level_re = re.compile('-p(?P<level>[0-9]+)')
+    comment_re = re.compile(r'\s+#.*$')
+    level_re = re.compile(r'-p(?P<level>[0-9]+)')
 
     @classmethod
     def read_series_file(cls, seriesfile):
